#include "PWM_Status.h"
PWM_Status::PWM_Status(const uint LED_Anode, const uint LED_Kathode, const uint slice_burst_Input, 
                         const uint16_t Wiederholung, uint16_t Level )
: _LED_Anode(LED_Anode),
  _LED_Kathode(LED_Kathode),
  _slice_burst_Input(slice_burst_Input),
  _slice_burst_wrap(Wiederholung),                                       // Das wird später überschrieben, aber erst in init, wenn Systemfrequenz bekannt ist
  _slice_burst_level(Level)
                                                                         // Hier folge ich Tipp4 aus dem Buch Effektiv C++ programmieren, 3. Auflage, von Scott Meyers
                                                                         // Dieser nutzt Initialisierungen statt Zuweisungen (innerhalb der geschweiften Klammer)
                                                                         // Die Logik dahinter ist, dass Datenelemente initialisiert werden bevor der Rumpf des Konstruktors betreten wird
                                                                         // Damit vermeidet man den Aufruf von Standardkonstruktoren, die dann gleich durch Zuweisungen überschrieben werden
                                                                         // Benötigt aber genausoviel Programm- und dynamischen Speicher.
{
  _slice_blink   = pwm_gpio_to_slice_num(LED_Anode);
  _channel_blink = pwm_gpio_to_channel(  LED_Anode);
  _slice_burst   = pwm_gpio_to_slice_num(LED_Kathode);
  _channel_burst = pwm_gpio_to_channel(  LED_Kathode);

}  

void PWM_Status::init() {
                                                                         // Ermittelt aus der Systemfrequenz die Blinkfrequenz (3,98 Hz aus 133 MHz, 1,44 Hz aus 48 MHz)
                                                                         // Wiederholung ist in Sekunden angegeben, daraus wird ermittelt, wie weit der zweite Counter hochgezählt werden muss
  uint32_t Systemfrequenz;
  float Blinkfrequenz;
  Systemfrequenz = clock_get_hz(clk_sys);
  Blinkfrequenz  = Systemfrequenz / (_slice_blink_divider );
  Blinkfrequenz  = Blinkfrequenz  / (_slice_blink_wrap * 2);
  _slice_burst_wrap = _slice_burst_wrap * Blinkfrequenz;

  gpio_set_function(       _LED_Anode,   GPIO_FUNC_PWM);
  gpio_set_drive_strength( _LED_Anode,   GPIO_DRIVE_STRENGTH_2MA);       // Erspart den Vorwiderstand, real ergbibt das 15mA mit roter LED. Definition ist dass bei 2mA-Strom noch 2,62V garantiert werden
  pwm_set_clkdiv_int_frac(_slice_blink, _slice_blink_divider, 0);        // Reduziert 125MHz auf 500kHz
  pwm_set_wrap(           _slice_blink, _slice_blink_wrap);              // Reduziert 500kHz auf 10Hz
  pwm_set_chan_level(     _slice_blink, _channel_blink, _slice_blink_level); // Tastgrad/Duty-Cycle wird hier eingestellt
  pwm_set_phase_correct(  _slice_blink,  true);                          // Hier verwendet um 10Hz auf 5Hz zu reduzieren
//  pwm_set_enabled(        _slice_blink,  true);                          // Blink wird erst mit start() enabled, damit zwei LED synchron blinken

  gpio_set_function(      _LED_Kathode,  GPIO_FUNC_PWM);
  gpio_set_drive_strength(_LED_Kathode,  GPIO_DRIVE_STRENGTH_2MA);       // Erspart den Vorwiderstand, weil Begrenzung bei Anode und Kathode ist der Strom nochmal niedriger als gegen GND
  gpio_set_function(_slice_burst_Input,  GPIO_FUNC_PWM);
  pwm_set_clkdiv_mode(    _slice_burst,  PWM_DIV_B_FALLING);             // Kanal B als Input
  pwm_set_clkdiv_int_frac(_slice_burst,  _slice_burst_divider, 0);       // Divider = 1, Zeile könnte auch entfallen
  pwm_set_wrap(           _slice_burst,  _slice_burst_wrap);             // Nach x Sekunden beginnt der Zyklus erneut
  pwm_set_chan_level(     _slice_burst,  _channel_burst, _slice_burst_level); // Dieser Wert kann später verändert werden um Anzahl zu variieren
  pwm_set_phase_correct(  _slice_burst,  false);                         // nicht benötigt, Zeile könnte auch entfallen
  pwm_set_output_polarity(_slice_burst,  true, true);                    // Sorgt dafür dass im "On"-Zustand der Output auf "Low" geht und die LED leuchtet
  pwm_set_enabled(        _slice_burst,  true);
}  

void PWM_Status::start() {
  pwm_set_enabled(        _slice_blink,  true);    
}

void PWM_Status::Status(uint16_t NewLevel) {                                     
  if (NewLevel > _slice_burst_wrap) _slice_burst_level = _slice_burst_wrap;
  else                              _slice_burst_level = NewLevel;
  pwm_set_chan_level(_slice_burst,  _channel_burst, _slice_burst_level);
}
